"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocaptureNetworkEvents = void 0;
const simple_in_memory_cache_1 = require("simple-in-memory-cache");
const with_simple_caching_1 = require("with-simple-caching");
const AppUsageEvent_1 = require("../../domain/AppUsageEvent");
const captureAppUsageEvent_1 = require("./captureAppUsageEvent");
/**
 * used to prevent infinite loop of logging on our own requests
 */
const APP_USAGE_EVENTS_EXPECTED_HOST_PREFIX = 'https://aue.';
const POLLING_NETWORK_REQUEST_HOST_PREFIXES = [
    'https://app.posthog.com/',
    'https://r.lr-ingest.io/',
];
/**
 * listen to and automatically capture network events
 *
 * note
 * - "with simple caching" to prevent duplicate mounting
 */
exports.autocaptureNetworkEvents = (0, with_simple_caching_1.withSimpleCaching)(() => {
    /**
     * network state has changed
     *
     * note
     * - online may have false positives, as it only checks that there is a network connection and not that the network is connected to the internet
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Window/online_event
     */
    window.addEventListener('offline', () => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.NETWORK,
        type: 'network.offline',
        details: null,
    }));
    window.addEventListener('online', () => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.NETWORK,
        type: 'network.online',
        details: null,
    }));
    /**
     * network request events
     *
     * note
     * - enables tracking duration of network requests
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Performance_API/Resource_timing
     */
    const observer = new PerformanceObserver((list) => {
        list.getEntries().forEach((entry) => {
            if (entry.duration &&
                !entry.name.includes(APP_USAGE_EVENTS_EXPECTED_HOST_PREFIX) &&
                // filter out the hosts that regularly poll // TODO: automatically detect these
                !POLLING_NETWORK_REQUEST_HOST_PREFIXES.some((hostPrefix) => entry.name.includes(hostPrefix)))
                void (0, captureAppUsageEvent_1.captureAppUsageEvent)({
                    source: AppUsageEvent_1.AppUsageEventSource.NETWORK,
                    type: 'network.request',
                    details: {
                        resource: entry.name,
                        milliseconds: Math.round(entry.duration),
                        bytes: {
                            transferred: entry.toJSON().transferSize,
                            encoded: entry.toJSON().encodedBodySize,
                            decoded: entry.toJSON().decodedBodySize,
                        },
                        effect: entry.toJSON().renderBlockingStatus,
                    },
                });
        });
    });
    observer.observe({ type: 'resource', buffered: true });
}, { cache: (0, simple_in_memory_cache_1.createCache)() });
//# sourceMappingURL=autocaptureNetworkEvents.js.map