"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocaptureRuntimeEvents = void 0;
const simple_in_memory_cache_1 = require("simple-in-memory-cache");
const with_simple_caching_1 = require("with-simple-caching");
const AppUsageEvent_1 = require("../../domain/AppUsageEvent");
const AppUsageEventEmissionError_1 = require("../AppUsageEventEmissionError");
const appUsageEventsEmissionQueue_1 = require("../emit/appUsageEventsEmissionQueue");
const captureAppUsageEvent_1 = require("./captureAppUsageEvent");
/**
 * listen to and automatically capture runtime events
 *
 * note
 * - "with simple caching" to prevent duplicate mounting
 */
exports.autocaptureRuntimeEvents = (0, with_simple_caching_1.withSimpleCaching)(() => {
    /**
     * script error events
     *
     * note
     * - fired on a Window object when a resource failed to load or couldn't be used — for example if a script has an execution error.
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Window/error_event
     */
    window.addEventListener('error', (event) => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.RUNTIME,
        type: 'runtime.error.resource',
        details: { errorType: event.type, errorMessage: event.message },
    }));
    /**
     * fullscreen error events
     *
     * note
     * - fired when the browser cannot switch to fullscreen mode.
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Document/fullscreenerror_event
     */
    window.addEventListener('fullscreenerror', () => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.RUNTIME,
        type: 'runtime.error.fullscreen',
        details: null, // TODO: figure out what details we may want
    }));
    /**
     * unhandled rejection events
     *
     * note
     * - fired when a rejected JavaScript Promise is handled late, i.e. when a handler is attached to the promise after its rejection had caused an unhandledrejection event.
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Window/unhandledrejection_event
     */
    window.addEventListener('unhandledrejection', (event) => event.reason instanceof Error &&
        event.reason.constructor.name === AppUsageEventEmissionError_1.AppUsageEventEmissionError.name // if its an app usage event emission error, dont capture it - as it would cause infinite loop
        ? null
        : (0, captureAppUsageEvent_1.captureAppUsageEvent)({
            source: AppUsageEvent_1.AppUsageEventSource.RUNTIME,
            type: 'runtime.rejection.unhandled',
            details: event.reason instanceof Error
                ? {
                    errorType: event.reason.constructor.name,
                    errorMessage: event.reason.message,
                }
                : { reason: event.reason.toString() },
        }));
    /**
     * rejection events
     *
     * note
     * - fired when a rejected JavaScript Promise is handled late, i.e. when a handler is attached to the promise after its rejection had caused an unhandledrejection event.
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Window/rejectionhandled_event
     */
    window.addEventListener('rejectionhandled', (event) => event.reason instanceof Error &&
        event.reason.constructor.name === AppUsageEventEmissionError_1.AppUsageEventEmissionError.name // if its an app usage event emission error, dont capture it - as it would cause infinite loop
        ? null
        : (0, captureAppUsageEvent_1.captureAppUsageEvent)({
            source: AppUsageEvent_1.AppUsageEventSource.RUNTIME,
            type: 'runtime.rejection.unhandled',
            details: event.reason instanceof Error
                ? {
                    errorType: event.reason.constructor.name,
                    errorMessage: event.reason.message,
                }
                : { reason: event.reason.toString() },
        }));
    /**
     * console logs
     *
     * note
     * - fires whenever a console.log/warn/error/etc call is made
     * - replaces the original console log implementation with ours, which runs the original implementation and captures the events
     *
     * ref
     * - https://stackoverflow.com/questions/9216441/intercept-calls-to-console-log-in-chrome
     */
    window.console = ((oldConsole) => {
        const createInterceptedConsoleMethod = ({ level, }) => {
            return (...args) => {
                console.log(args);
                // call the original log method
                oldConsole[level](...args);
                // stop here if this was a console log produced by our app-usage-events emission queue
                if (args[0] === appUsageEventsEmissionQueue_1.APP_USAGE_EVENTS_EMISSION_LOG_FIRST_ARG)
                    return;
                // if (args[0].toString().includes(App))
                // capture the event
                void (0, captureAppUsageEvent_1.captureAppUsageEvent)({
                    source: AppUsageEvent_1.AppUsageEventSource.RUNTIME,
                    type: `runtime.console.${level}`,
                    details: { args },
                });
            };
        };
        return Object.assign({}, oldConsole);
    })(window.console);
}, { cache: (0, simple_in_memory_cache_1.createCache)() });
//# sourceMappingURL=autocaptureRuntimeEvents.js.map