"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findOrCreateAppUsageSession = exports.SESSION_LAST_USE_KEY = void 0;
const date_fns_1 = require("date-fns");
const domain_objects_1 = require("domain-objects");
const simple_in_memory_cache_1 = require("simple-in-memory-cache");
const type_fns_1 = require("type-fns");
const uuid_1 = require("uuid");
const with_simple_caching_1 = require("with-simple-caching");
const AppUsageBrowser_1 = require("../../domain/AppUsageBrowser");
const AppUsageSession_1 = require("../../domain/AppUsageSession");
const queueTaskEmitToRemote_1 = require("../emit/asyncTask/queueTaskEmitToRemote");
const base64EncodeCompressedPayload_1 = require("../emit/compression/base64EncodeCompressedPayload");
const serializeAndCompressPayload_1 = require("../emit/compression/serializeAndCompressPayload");
const findOrCreateInstallationUuid_1 = require("./findOrCreateInstallationUuid");
const getApplicationOfSession_1 = require("./getApplicationOfSession");
const SESSION_STORAGE_KEY = 'app-usage-session';
exports.SESSION_LAST_USE_KEY = 'app-usage-session-lastseenat';
/**
 * finds or creates the session for this app-usage-session
 *
 * note
 * - persists the session in localstorage, for lookup
 * - replaces the session if session specific data has changed
 * - wrapped with simple caching to prevent redundant calculations
 */
exports.findOrCreateAppUsageSession = (0, with_simple_caching_1.withSimpleCaching)(() => __awaiter(void 0, void 0, void 0, function* () {
    // define what a new session would look like
    const sessionNew = new AppUsageSession_1.AppUsageSession({
        uuid: (0, uuid_1.v4)(),
        installationUuid: yield (0, findOrCreateInstallationUuid_1.findOrCreateInstallationUuid)(),
        application: (0, getApplicationOfSession_1.getApplicationOfSession)(),
        device: null,
        network: null,
        browser: new AppUsageBrowser_1.AppUsageBrowser({
            userAgentClaim: window.navigator.userAgent,
        }),
    });
    // check if a session exists already
    const sessionFoundJSON = localStorage.getItem(SESSION_STORAGE_KEY);
    const sessionFound = sessionFoundJSON
        ? (0, domain_objects_1.deserialize)(sessionFoundJSON, {
            with: [AppUsageSession_1.AppUsageSession],
        })
        : null;
    const sessionLastUsedAtJSON = localStorage.getItem(exports.SESSION_LAST_USE_KEY);
    const sessionLastUsedAt = sessionLastUsedAtJSON
        ? (() => {
            const { sessionUuid, lastUsedAtMse } = JSON.parse(sessionLastUsedAtJSON);
            // if the last used at mse was recorded for a different session, then it is not applicable here
            if ((sessionFound === null || sessionFound === void 0 ? void 0 : sessionFound.uuid) !== sessionUuid)
                return null;
            // otherwise, its applicable
            return lastUsedAtMse;
        })()
        : null;
    // evaluate if found session is still valid, if one was found
    const sessionFoundHasSameStaticData = sessionFound &&
        (0, domain_objects_1.serialize)((0, type_fns_1.omit)(sessionNew, ['uuid'])) ===
            (0, domain_objects_1.serialize)((0, type_fns_1.omit)(sessionFound, ['uuid']));
    const sessionFoundUsedWithinLastHour = sessionFound &&
        sessionLastUsedAt &&
        Math.abs((0, date_fns_1.differenceInHours)(sessionLastUsedAt, new Date().getTime())) <= 1;
    // if the session exists and is still valid, return it
    if (
    // session exists
    sessionFound &&
        // session static data is the same
        sessionFoundHasSameStaticData &&
        // session was last used within an hour
        sessionFoundUsedWithinLastHour)
        return sessionFound;
    // otherwise, save and return the new session
    yield (0, queueTaskEmitToRemote_1.queueTaskEmitToRemote)({
        endpoint: '/capture/session',
        payload: (0, base64EncodeCompressedPayload_1.base64EncodeCompressedPayload)((0, serializeAndCompressPayload_1.serializeAndCompressPayload)(sessionNew)),
    });
    localStorage.setItem(SESSION_STORAGE_KEY, (0, domain_objects_1.serialize)(sessionNew));
    return sessionNew;
}), {
    cache: (0, simple_in_memory_cache_1.createCache)({
        defaultSecondsUntilExpiration: 5 * 60, // check max every 5 min
    }),
});
//# sourceMappingURL=findOrCreateAppUsageSession.js.map