"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.autocaptureActivityEvents = void 0;
const simple_in_memory_cache_1 = require("simple-in-memory-cache");
const simple_in_memory_queue_1 = require("simple-in-memory-queue");
const type_fns_1 = require("type-fns");
const with_simple_caching_1 = require("with-simple-caching");
const AppUsageEvent_1 = require("../../domain/AppUsageEvent");
const captureAppUsageEvent_1 = require("./captureAppUsageEvent");
const getDescriptionOfElement_1 = require("./dom/getDescriptionOfElement");
const keyboardEventsQueue = (0, simple_in_memory_queue_1.createQueueWithDebounceConsumer)({
    gap: { milliseconds: 500 },
    consumer: ({ items }) => __awaiter(void 0, void 0, void 0, function* () {
        // calculate the duration
        const duration = Math.round(items.slice(-1)[0].occurredAtMse - items[0].occurredAtMse);
        // if change, capture the event
        yield (0, captureAppUsageEvent_1.captureAppUsageEvent)({
            source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
            type: 'activity.keyboard',
            details: {
                duration,
                // !: note: we do not capture the actual keys, to protect privacy and for security... otherwise, that'd be a keylogger
                keysDown: items.filter((item) => item.mode === 'down').length,
                keysUp: items.filter((item) => item.mode === 'up').length,
            },
        });
    }),
});
const pointerPressEventsQueue = (0, simple_in_memory_queue_1.createQueueWithDebounceConsumer)({
    gap: { milliseconds: 300 },
    consumer: ({ items }) => __awaiter(void 0, void 0, void 0, function* () {
        // define the sequence of events, merging up+down w/ same target+pointer as "press"
        const sequence = (0, simple_in_memory_queue_1.createQueue)({
            order: simple_in_memory_queue_1.QueueOrder.LAST_IN_FIRST_OUT,
        });
        const getMergeKey = (item) => JSON.stringify((0, type_fns_1.pick)(item, ['pointer', 'target']));
        for (const item of items) {
            // handle pointer down
            if (item.mode === 'down') {
                sequence.push(item);
                continue;
            }
            // handle pointer up, when it was last down on the same thing -> convert to "press"
            const lastItem = sequence.peek()[0];
            if ((lastItem === null || lastItem === void 0 ? void 0 : lastItem.mode) === 'down' &&
                getMergeKey(lastItem) === getMergeKey(item)) {
                sequence.pop(); // drop that last "down"
                sequence.push(Object.assign(Object.assign({}, item), { mode: 'press' })); // and replace it with a "press"
                continue;
            }
            // handle pointer up, otherwise
            sequence.push(item);
        }
        // emit an event for each in sequence, post merging
        yield Promise.all(sequence
            .pop(sequence.length)
            .reverse()
            .map((item) => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
            source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
            type: `activity.pointer.${item.mode}`,
            details: {
                pointer: item.pointer,
                position: item.pointer,
                target: item.target,
            },
        })));
    }),
});
const pointerMoveEventsQueue = (0, simple_in_memory_queue_1.createQueueWithDebounceConsumer)({
    gap: { milliseconds: 300 },
    consumer: ({ items }) => __awaiter(void 0, void 0, void 0, function* () {
        const eventsPerPointer = items.reduce((summary, item) => {
            var _a;
            const key = JSON.stringify(item.pointer);
            const current = (_a = summary[key]) !== null && _a !== void 0 ? _a : [];
            return Object.assign(Object.assign({}, summary), { [key]: [...current, item] });
        }, {});
        // TODO: send separate events for changes in direction in motion (i.e., one event for moving down, another for suddenly changing back to up?)
        yield Promise.all(Object.values(eventsPerPointer).map((theseItems) => {
            // calculate the duration
            const duration = Math.round(theseItems.slice(-1)[0].occurredAtMse - theseItems[0].occurredAtMse);
            // calculate the difference in position
            const start = theseItems[0].position;
            const end = theseItems.slice(-1)[0].position;
            const change = { x: end.x - start.x, y: end.y - start.y };
            // if no change, no event
            if (!change.x && !change.y)
                return null;
            // if change, capture the event
            return (0, captureAppUsageEvent_1.captureAppUsageEvent)({
                source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
                type: 'activity.pointer.move',
                details: {
                    pointer: theseItems[0].pointer,
                    duration,
                    position: {
                        start,
                        end,
                        change,
                    },
                },
            });
        }));
    }),
});
const scrollEventsQueue = (0, simple_in_memory_queue_1.createQueueWithDebounceConsumer)({
    gap: { milliseconds: 300 },
    consumer: ({ items }) => __awaiter(void 0, void 0, void 0, function* () {
        // calculate the duration
        const duration = Math.round(items.slice(-1)[0].occurredAtMse - items[0].occurredAtMse);
        // calculate the difference in position
        const start = items[0].position;
        const end = items.slice(-1)[0].position;
        const change = { x: end.x - start.x, y: end.y - start.y };
        // if no change, no event
        if (!change.x && !change.y)
            return;
        // if change, capture the event
        yield (0, captureAppUsageEvent_1.captureAppUsageEvent)({
            source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
            type: 'activity.scroll',
            details: {
                duration,
                position: {
                    start,
                    end,
                    change,
                },
            },
        });
    }),
});
const selectionEventsQueue = (0, simple_in_memory_queue_1.createQueueWithDebounceConsumer)({
    gap: { milliseconds: 300 },
    consumer: ({ items }) => __awaiter(void 0, void 0, void 0, function* () {
        // calculate the duration
        const duration = Math.round(items.slice(-1)[0].occurredAtMse - items[0].occurredAtMse);
        // if change, capture the event
        yield (0, captureAppUsageEvent_1.captureAppUsageEvent)({
            source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
            type: 'activity.selection.change',
            details: {
                duration,
                selection: items.slice(-1)[0].selection, // only report final selection
            },
        });
    }),
});
/**
 * listen to and automatically capture network events
 *
 * note
 * - "with simple caching" to prevent duplicate mounting
 */
exports.autocaptureActivityEvents = (0, with_simple_caching_1.withSimpleCaching)(() => {
    /**
     * keyboard events
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent
     */
    window.addEventListener('keydown', (event) => keyboardEventsQueue.push({
        occurredAtMse: event.timeStamp,
        mode: 'down',
        key: event.key,
        isComposing: event.isComposing,
    }));
    window.addEventListener('keyup', (event) => keyboardEventsQueue.push({
        occurredAtMse: event.timeStamp,
        mode: 'up',
        key: event.key,
        isComposing: event.isComposing,
    }));
    /**
     * pointer events (mouse, touch, etc)
     *
     * note
     * - pointer events handle mouse, touch and pen inputs at the same time
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Pointer_events/Using_Pointer_Events
     * - https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent
     * - https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent
     * - https://stackoverflow.com/questions/6073505/what-is-the-difference-between-screenx-y-clientx-y-and-pagex-y
     *
     * TODO: combine pointer events into gestures
     */
    window.addEventListener('pointermove', (event) => pointerMoveEventsQueue.push({
        pointer: { id: event.pointerId, type: event.pointerType },
        occurredAtMse: event.timeStamp,
        position: { x: event.pageX, y: event.pageY },
    }));
    window.addEventListener('pointerdown', (event) => pointerPressEventsQueue.push({
        occurredAtMse: event.timeStamp,
        mode: 'down',
        pointer: { id: event.pointerId, type: event.pointerType },
        position: { x: event.pageX, y: event.pageY },
        target: event.target instanceof Element
            ? (0, getDescriptionOfElement_1.getDescriptionOfElement)(event.target)
            : undefined,
    }));
    window.addEventListener('pointerup', (event) => pointerPressEventsQueue.push({
        occurredAtMse: event.timeStamp,
        mode: 'up',
        pointer: { id: event.pointerId, type: event.pointerType },
        position: { x: event.pageX, y: event.pageY },
        target: event.target instanceof Element
            ? (0, getDescriptionOfElement_1.getDescriptionOfElement)(event.target)
            : undefined,
    }));
    /**
     * copy, cut, paste events
     *
     * note
     * - fired when the user initiates a copy, cut, or paste action through the browser's user interface.
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Window/copy_event
     * - https://developer.mozilla.org/en-US/docs/Web/API/Window/cut_event
     * - https://developer.mozilla.org/en-US/docs/Web/API/Window/paste_event
     */
    window.addEventListener('copy', () => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
        type: 'activity.clipboard.copy',
        details: null,
    }));
    window.addEventListener('cut', () => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
        type: 'activity.clipboard.cut',
        details: null,
    }));
    window.addEventListener('paste', () => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
        type: 'activity.clipboard.paste',
        details: null,
    }));
    /**
     * full screen change
     *
     * note
     * - fired when
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Document/fullscreenchange_event
     */
    document.addEventListener('fullscreenchange', () => (0, captureAppUsageEvent_1.captureAppUsageEvent)({
        source: AppUsageEvent_1.AppUsageEventSource.ACTIVITY,
        type: 'activity.fullscreen.change',
        details: { fullscreen: !!document.fullscreenElement },
    }));
    /**
     * scroll events
     *
     * note
     * - fires when the document view has been scrolled
     * - we must debounce these events and merge them into one summary, as otherwise it is too noisy.
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Document/scroll_event
     *
     * TODO: use `scrollend` event when it has mainstream support
     */
    document.addEventListener('scroll', (event) => scrollEventsQueue.push({
        occurredAtMse: event.timeStamp,
        position: { x: window.scrollX, y: window.scrollY },
    }));
    /**
     * selection events
     *
     * note
     * - fired when the current Selection of a Document is changed.
     *
     * ref
     * - https://developer.mozilla.org/en-US/docs/Web/API/Document/selectionchange_event
     */
    document.addEventListener('selectionchange', (event) => window.getSelection() && window.getSelection().toString()
        ? selectionEventsQueue.push({
            occurredAtMse: event.timeStamp,
            selection: window.getSelection().toString(),
        })
        : null);
}, { cache: (0, simple_in_memory_cache_1.createCache)() });
//# sourceMappingURL=autocaptureActivityEvents.js.map